<!doctype html>
<head>
	<meta charset="utf-8">
	<meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1">
	<title>{% if title %}{{ title }}{% else %}Brightway2 Web UI{% endif %}</title>
	<link rel="shortcut icon" type="image/ico" href="{{ url_for('static', filename="img/favicon.ico") }}" />
	<meta name="description" content="The web user interface for the Brightway2 LCA engine">
	<meta name="viewport" content="width=device-width">
	<link rel="stylesheet" href="{{ url_for('static', filename="blueprint/screen.css") }}" type="text/css" media="screen, projection">
	<link rel="stylesheet" href="{{ url_for('static', filename="blueprint/plugins/buttons/screen.css") }}" type="text/css" media="screen, projection">
	<link rel="stylesheet" href="{{ url_for('static', filename="blueprint/print.css") }}" type="text/css" media="print">
	<link rel="stylesheet" href="{{ url_for('static', filename="backgrid/backgrid.css") }}" type="text/css" media="screen, projection">
	<!--[if lt IE 8]><link rel="stylesheet" href="{{ url_for('static', filename="blueprint/ie.css") }}" type="text/css" media="screen, projection"><![endif]-->
	<link rel="stylesheet" href="{{ url_for('static', filename="css/base.css") }}" type="text/css" media="screen, projection">
	<link href='http://fonts.googleapis.com/css?family=Source+Sans+Pro' rel='stylesheet' type='text/css'>
	<script src="https://ajax.googleapis.com/ajax/libs/jquery/1.8.3/jquery.min.js"></script>
	<script src="{{ url_for('static', filename="backgrid/backgrid.js") }}"></script>
	<script src="{{ url_for('static', filename="backgrid/backgrid-table.js") }}"></script>
	<script src="https://ajax.googleapis.com/ajax/libs/jqueryui/1.9.2/jquery-ui.min.js"></script>
	<!-- <script src="https://cdnjs.cloudflare.com/ajax/libs/datatables/1.10.21/jquery.dataTables.min.js"></script> -->
	<!-- <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/datatables/1.10.21/css/jquery.dataTables.css" type="text/css" media="screen, projection"> -->
	<link rel="stylesheet" href="https://ajax.googleapis.com/ajax/libs/jqueryui/1.9.0/themes/flick/jquery-ui.css" type="text/css" media="screen, projection">
	<script src="http://d3js.org/d3.v3.min.js"></script>
	{% block extrahead %}{% endblock %}
</head>
<body>
	<div class="container">
		{% block header %}<h1><span id="brightway">Brightway</span><span id="two">2</span> web</h1><hr>{% endblock %}
		{% block body %}{% endblock %}
	</div>
</body>
</html>
